A Calibration Scheme for Non-Line-of-Sight Sensing Setups
=========================================================

Version 1.1 (19.07.2020)

This folder contains the python implementation of the NLoS setup calibration method that is described in "A Calibration Scheme for Non-Line-of-Sight Sensing Setups". The source code is published to help other researches gain a deeper understanding of the inner workings of our method and also to enable them to calibrate their own setups.
NLoS imaging can be performed with a wide variety of hardware, each with their own file formats. Therefore, users will have to write their own data conversion routines to use this framework. We show the calibration of a synthetic scene which should give a good understanding on how the framework is supposed to be used and help researchers to plug in their own data.
The calibration output is stored in machine-readable json files which can be imported intod NLoS reconstruction frameworks.


Prerequisites
-------------

The framework is implemented in Python and relies heavily on the scipy stack. A number of other standard python libraries is used as well. Additional libraries:

- autograd (https://github.com/HIPS/autograd)
- ClayFlannigan's ICP implementation (https://github.com/ClayFlannigan/icp), bundled
- recordclass (https://pypi.org/project/recordclass/)
- tabulate (https://pypi.org/project/tabulate/)

All dependencies can be installed via pip and/or conda.


Overview
--------

A good starting point is `main.py`. When executed, it calibrates a synthetic scene and displays the results. Comments in the code explain what is happening in each step.

Other important parts of the code:

- `model.Setup` defines the geometry of a NLoS setup. This is the main input and output data structure for the calibration.
- `plotting.py` contains function to plot these setup structures.
- `optimization.py` contains the main optimization code and implements the three different parametrizations.
- `postpro.least_squares_transform` finds the best transformation between two setups (e.g. ground truth and calibration) to make them comparable.