import autograd.numpy as np
from autograd import grad, jacobian
import matplotlib.pyplot as plt


# returns a matrix transforming points from p to q. H*p = q
def compute_homography(p, q):
	assert(p.shape==(4,2))
	assert(q.shape==(4,2))

	# build an equation system and solve it
	M = np.array([
		[-p[0,0], -p[0,1], -1, 0,       0,        0, p[0,0]*q[0,0], p[0,1]*q[0,0], q[0,0]],
		[0,        0,       0, -p[0,0], -p[0,1], -1, p[0,0]*q[0,1], p[0,1]*q[0,1], q[0,1]],

		[-p[1,0], -p[1,1], -1, 0,       0,        0, p[1,0]*q[1,0], p[1,1]*q[1,0], q[1,0]],
		[0,        0,       0, -p[1,0], -p[1,1], -1, p[1,0]*q[1,1], p[1,1]*q[1,1], q[1,1]],

		[-p[2,0], -p[2,1], -1, 0,       0,        0, p[2,0]*q[2,0], p[2,1]*q[2,0], q[2,0]],
		[0,        0,       0, -p[2,0], -p[2,1], -1, p[2,0]*q[2,1], p[2,1]*q[2,1], q[2,1]],

		[-p[3,0], -p[3,1], -1, 0,       0,        0, p[3,0]*q[3,0], p[3,1]*q[3,0], q[3,0]],
		[0,        0,       0, -p[3,0], -p[3,1], -1, p[3,0]*q[3,1], p[3,1]*q[3,1], q[3,1]],

		[0, 0, 0, 0, 0, 0, 0, 0, 1]
	])

	X = np.array([0, 0, 0, 0, 0, 0, 0, 0, 1])

	H = np.linalg.solve(M, X)
	return H.reshape((3,3))

def homographic_transform(H, p):
	assert(H.shape == (3,3))
	assert(p.shape[-1] == 2)

	if p.shape == (2,):
		q = np.array([p[0], p[1], 1])
		t = np.dot(H, q)
		t /= t[2]
		return t[:2]
	else:
		result = list()
		for i in range(len(p)):
			q = np.array([p[i, 0], p[i, 1], 1])
			t = np.dot(H, q)
			t /= t[2]
			result.append(t[:2])
		return np.array(result)

def f(x):
	p = np.array([[0, 0], [0, 1], [1, 0], [1, 1]])
	q = np.array([[0, x[0]], [2, x[1]], [x[2], 2], [12, 11]])
	H = compute_homography(p, q)
	return np.linalg.norm(homographic_transform(H, np.array([2, 1])))



def plot():
	p = np.array([[0, 0], [0, 1], [1, 0], [1, 1]])
	q = np.array([[0, 3], [2, 10], [9, 2], [12, 11]])

	plt.scatter(p[:,0], p[:,1])
	plt.scatter(q[:,0], q[:,1])
	plt.show()

	H = compute_homography(p, q)
	print(homographic_transform(H, p))



if __name__ == "__main__":
	x = np.array([3.0, 10.0, 9.0])
	print(f(x))
	print(jacobian(f)(x))
	print(grad(f)(x))
	plot()