
import autograd.numpy as np

def reflect_point_on_plane(point, plane):
	# plane ix n_x, n_y, n_z, d
	n = plane[:-1]
	d = plane[-1]
	x = np.dot(n, point) + d
	ref = point - 2*n*x
	return ref


def point_of_reflection_on_plane(p1, p2, plane):
	p1_prime = reflect_point_on_plane(p1, plane)
	# ray/plane intersection of p1_prime -> p2 and plane:
	n = plane[:-1]
	d = plane[-1]
	y = - (np.dot(n, p1_prime)+d) / np.dot(n, (p2-p1_prime)) # distance on the ray

	return p1_prime + y*(p2-p1_prime) # evaluate ray equation


def distance_to_plane(point, plane):
	n = plane[:-1]
	d = plane[-1]
	x = np.dot(n, point) + d
	return x

# orthogonally projects point onto plane
def nearest_point_on_plane(point, plane):
	n = plane[:-1]
	return point - n*distance_to_plane(point, plane)


def create_3d_plane(p1, p2, p3):
	normal = np.cross(p1-p2, p1-p3)
	normal = normal / np.linalg.norm(normal)
	dist = -np.dot(normal, p1)
	return np.hstack((normal, dist))


def two_points_on_plane(plane):
	n = plane[:-1]
	d = plane[-1]

	m = np.array([n[1], -n[0]])

	p0 = n*(-d)
	p1 = p0 + m*12

	return np.array([p0, p1])


def three_points_on_plane(plane):
	# project 4 canonical vectors on the plane and return three unique results
	p1 = nearest_point_on_plane([0, 0, 0], plane)
	p2 = nearest_point_on_plane([1, 0, 0], plane)
	p3 = nearest_point_on_plane([0, 1, 0], plane)
	p4 = nearest_point_on_plane([0, 0, 1], plane)

	# this removes one point, if the plane is perpendicular to one of the canonical basis vectors
	s = np.vstack((p1, p2, p3, p4))
	s = np.unique(s, axis=0)
	return s[:3]


def normalize_plane(plane):
	n = plane[:-1]
	d = plane[-1]

	x = np.linalg.norm(n)
	n = n / x
	d = d * x
	return np.hstack((n, d))
