import numpy as np
import math_util
from sklearn.decomposition import PCA # conda install scikit-learn
import icp
from model import Setup



def transform_points(transf, points):
	assert(points.shape[1]==3)
	d = np.ones((points.shape[0], 4))
	d[:, 0:3] = points
	d = np.dot(transf, d.T).T
	return d[:, 0:3]


# returns a transformed reconstruction that fits the reference as close as possible (L2-norm)
def least_squares_transform(reference, reconstruction):
	r_S, r_L, r_C, r_M, _ = reference
	i_S, i_L, i_C, i_M, _ = reconstruction

	reference_points = np.vstack((r_S, r_L, r_C))
	reconstruction_points = np.vstack((i_S, i_L, i_C))

	# just as ICP but without recalculating correspondences
	T, _, _ = icp.best_fit_transform(reconstruction_points, reference_points)

	transformed_setup = Setup(
		transform_points(T, i_S),
		transform_points(T, i_L),
		transform_points(T, i_C),
		i_M,
		reconstruction.connection_list
	)

	return transformed_setup
